# encoding: utf-8

"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

from __future__ import absolute_import

from datetime import datetime
from textwrap import dedent

import six

from .common import PingTestData


# ping google.com -q -c 60:
#   - Debian 8.2 w/ iputils-ping 20121221-5+b2
PING_DEBIAN_SUCCESS_0 = six.b(
    dedent(
        """\
    PING google.com (216.58.196.238) 56(84) bytes of data.

    --- google.com ping statistics ---
    60 packets transmitted, 60 received, 0% packet loss, time 59153ms
    rtt min/avg/max/mdev = 61.425/99.731/212.597/27.566 ms
    """
    )
)

# Ubuntu stretch
#   $ ping -c 5 twitter.com
PING_UBUNTU_SUCCESS_0 = six.b(
    dedent(
        """\
    PING twitter.com (104.244.42.65) 56(84) bytes of data.
    64 bytes from 104.244.42.65: icmp_seq=1 ttl=53 time=68.5 ms
    64 bytes from 104.244.42.65: icmp_seq=2 ttl=53 time=67.7 ms
    64 bytes from 104.244.42.65: icmp_seq=3 ttl=53 time=65.6 ms
    64 bytes from 104.244.42.65: icmp_seq=4 ttl=53 time=65.6 ms
    64 bytes from 104.244.42.65: icmp_seq=5 ttl=53 time=64.9 ms

    --- twitter.com ping statistics ---
    5 packets transmitted, 5 received, 0% packet loss, time 4003ms
    rtt min/avg/max/mdev = 64.912/66.494/68.524/1.430 ms
    """
    )
)

#   $ ping -c 5 google.com -D
PING_UBUNTU_SUCCESS_1 = dedent(
    """\
    PING google.com (74.125.24.100) 56(84) bytes of data.
    [1524930937.003555] 64 bytes from 74.125.24.100: icmp_seq=1 ttl=39 time=148 ms
    [1524930937.787175] 64 bytes from 74.125.24.100: icmp_seq=2 ttl=39 time=137 ms
    [1524930938.787642] 64 bytes from 74.125.24.100: icmp_seq=3 ttl=39 time=137 ms
    [1524930939.787653] 64 bytes from 74.125.24.100: icmp_seq=4 ttl=39 time=136 ms
    [1524930940.788365] 64 bytes from 74.125.24.100: icmp_seq=5 ttl=39 time=136 ms

    --- google.com ping statistics ---
    5 packets transmitted, 5 received, 0% packet loss, time 4001ms
    rtt min/avg/max/mdev = 136.537/139.174/148.006/4.425 ms
    """
)

DEBIAN_SUCCESS_0 = PingTestData(
    PING_DEBIAN_SUCCESS_0,
    {
        "destination": "google.com",
        "packet_transmit": 60,
        "packet_receive": 60,
        "packet_loss_count": 0,
        "packet_loss_rate": 0.0,
        "packet_duplicate_rate": 0,
        "packet_duplicate_count": 0,
        "rtt_min": 61.425,
        "rtt_avg": 99.731,
        "rtt_max": 212.597,
        "rtt_mdev": 27.566,
    },
    [],
)
UBUNTU_SUCCESS_0 = PingTestData(
    PING_UBUNTU_SUCCESS_0,
    {
        "destination": "twitter.com",
        "packet_transmit": 5,
        "packet_receive": 5,
        "packet_loss_count": 0,
        "packet_loss_rate": 0.0,
        "packet_duplicate_rate": 0,
        "packet_duplicate_count": 0,
        "rtt_min": 64.912,
        "rtt_avg": 66.494,
        "rtt_max": 68.524,
        "rtt_mdev": 1.430,
    },
    [
        {"timestamp": None, "icmp_seq": 1, "ttl": 53, "time": 68.5, "duplicate": False},
        {"timestamp": None, "icmp_seq": 2, "ttl": 53, "time": 67.7, "duplicate": False},
        {"timestamp": None, "icmp_seq": 3, "ttl": 53, "time": 65.6, "duplicate": False},
        {"timestamp": None, "icmp_seq": 4, "ttl": 53, "time": 65.6, "duplicate": False},
        {"timestamp": None, "icmp_seq": 5, "ttl": 53, "time": 64.9, "duplicate": False},
    ],
)
UBUNTU_SUCCESS_1 = PingTestData(
    PING_UBUNTU_SUCCESS_1,
    {
        "destination": "google.com",
        "packet_transmit": 5,
        "packet_receive": 5,
        "packet_loss_count": 0,
        "packet_loss_rate": 0.0,
        "packet_duplicate_rate": 0,
        "packet_duplicate_count": 0,
        "rtt_min": 136.537,
        "rtt_avg": 139.174,
        "rtt_max": 148.006,
        "rtt_mdev": 4.425,
    },
    [
        {
            "timestamp": datetime(2018, 4, 29, 0, 55, 37),
            "icmp_seq": 1,
            "ttl": 39,
            "time": 148.0,
            "duplicate": False,
        },
        {
            "timestamp": datetime(2018, 4, 29, 0, 55, 37),
            "icmp_seq": 2,
            "ttl": 39,
            "time": 137.0,
            "duplicate": False,
        },
        {
            "timestamp": datetime(2018, 4, 29, 0, 55, 38),
            "icmp_seq": 3,
            "ttl": 39,
            "time": 137.0,
            "duplicate": False,
        },
        {
            "timestamp": datetime(2018, 4, 29, 0, 55, 39),
            "icmp_seq": 4,
            "ttl": 39,
            "time": 136.0,
            "duplicate": False,
        },
        {
            "timestamp": datetime(2018, 4, 29, 0, 55, 40),
            "icmp_seq": 5,
            "ttl": 39,
            "time": 136.0,
            "duplicate": False,
        },
    ],
)

WINDOWS7SP1_SUCCESS = PingTestData(
    # ping google.com -n 10:
    dedent(
        """\
        Pinging google.com [216.58.196.238] with 32 bytes of data:
        Reply from 216.58.196.238: bytes=32 time=87ms TTL=51
        Reply from 216.58.196.238: bytes=32 time=97ms TTL=51
        Reply from 216.58.196.238: bytes=32 time=56ms TTL=51
        Reply from 216.58.196.238: bytes=32 time=95ms TTL=51
        Reply from 216.58.196.238: bytes=32 time=194ms TTL=51
        Reply from 216.58.196.238: bytes=32 time=98ms TTL=51
        Reply from 216.58.196.238: bytes=32 time=93ms TTL=51
        Reply from 216.58.196.238: bytes=32 time=96ms TTL=51
        Reply from 216.58.196.238: bytes=32 time=96ms TTL=51
        Reply from 216.58.196.238: bytes=32 time=165ms TTL=51

        Ping statistics for 216.58.196.238:
            Packets: Sent = 10, Received = 10, Lost = 0 (0% loss),
        Approximate round trip times in milli-seconds:
            Minimum = 56ms, Maximum = 194ms, Average = 107ms
        """
    ),
    {
        "destination": "216.58.196.238",
        "packet_transmit": 10,
        "packet_receive": 10,
        "packet_loss_count": 0,
        "packet_loss_rate": 0.0,
        "packet_duplicate_count": None,
        "packet_duplicate_rate": None,
        "rtt_min": 56,
        "rtt_avg": 107,
        "rtt_max": 194,
        "rtt_mdev": None,
    },
    [],
)



#!/usr/bin/env python
# encoding: utf-8

"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

from __future__ import print_function

import json
import sys
from textwrap import dedent

import pingparsing


def main():
    parser = pingparsing.PingParsing()
    stats = parser.parse(
        dedent(
            """\
        PING google.com (74.125.24.100) 56(84) bytes of data.
        [1524930937.003555] 64 bytes from 74.125.24.100: icmp_seq=1 ttl=39 time=148 ms
        [1524930937.787175] 64 bytes from 74.125.24.100: icmp_seq=2 ttl=39 time=137 ms
        [1524930938.787642] 64 bytes from 74.125.24.100: icmp_seq=3 ttl=39 time=137 ms
        [1524930939.787653] 64 bytes from 74.125.24.100: icmp_seq=4 ttl=39 time=136 ms
        [1524930940.788365] 64 bytes from 74.125.24.100: icmp_seq=5 ttl=39 time=136 ms

        --- google.com ping statistics ---
        5 packets transmitted, 5 received, 0% packet loss, time 4001ms
        rtt min/avg/max/mdev = 136.537/139.174/148.006/4.425 ms
        """
        )
    )

    print("[ping statistics]")
    print(json.dumps(stats.as_dict(), indent=4))

    print("\n[icmp reply]")
    for icmp_reply in stats.icmp_reply_list:
        print(icmp_reply)

    return 0


if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python
# encoding: utf-8

"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

from __future__ import print_function

import json
import sys

import pingparsing


def main():
    ping_parser = pingparsing.PingParsing()
    transmitter = pingparsing.PingTransmitter()
    transmitter.destination_host = "google.com"
    transmitter.count = 10
    result = transmitter.ping()

    print(json.dumps(ping_parser.parse(result).as_dict(), indent=4))

    return result.returncode


if __name__ == "__main__":
    sys.exit(main())



# encoding: utf-8

"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

import pytest
from pingparsing import PingTransmitter
from typepy import RealNumber

from .common import ping_parser  # noqa: W0611


@pytest.fixture
def transmitter():
    return PingTransmitter()


class Test_PingTransmitter_ping(object):
    @pytest.mark.xfail(run=False)
    @pytest.mark.parametrize(["host", "deadline"], [["localhost", 1], ["127.0.0.1", 1], ["::1", 1]])
    def test_normal_deadline(self, transmitter, host, deadline):
        transmitter.destination_host = host
        transmitter.deadline = deadline
        result = transmitter.ping()

        assert result.returncode == 0
        assert len(result.stdout) > 0

    @pytest.mark.xfail(run=False)
    @pytest.mark.parametrize(
        ["host", "count", "deadline"], [["localhost", 1, None], ["localhost", 1, 1000]]
    )
    def test_normal_count(self, transmitter, host, count, deadline):
        transmitter.destination_host = host
        transmitter.deadline = deadline
        transmitter.count = count
        result = transmitter.ping()

        assert result.returncode == 0
        assert len(result.stdout) > 0

    @pytest.mark.xfail(run=False)
    @pytest.mark.parametrize(["host", "count"], [["localhost", 3]])
    def test_normal_send_parse(self, transmitter, ping_parser, host, count):
        transmitter.destination_host = host
        transmitter.count = count
        result = transmitter.ping()

        ping_parser.parse(result.stdout)

        assert ping_parser.packet_transmit >= count
        assert RealNumber(ping_parser.packet_receive).is_type()
        assert RealNumber(ping_parser.packet_loss_rate).is_type()
        assert RealNumber(ping_parser.packet_loss_count).is_type()
        assert RealNumber(ping_parser.packet_duplicate_rate).is_type()
        assert RealNumber(ping_parser.packet_duplicate_count).is_type()
        assert RealNumber(ping_parser.rtt_min).is_type()
        assert RealNumber(ping_parser.rtt_avg).is_type()
        assert RealNumber(ping_parser.rtt_max).is_type()
        assert RealNumber(ping_parser.rtt_mdev).is_type()

    @pytest.mark.parametrize(
        ["host", "deadline", "expected"],
        [
            ["", 1, ValueError],
            ["localhost", 0, ValueError],
            ["localhost", -1, ValueError],
            ["localhost", "a", ValueError],
            [None, 1, ValueError],
        ],
    )
    def test_except_deadline(self, transmitter, host, deadline, expected):
        transmitter.destination_host = host
        transmitter.deadline = deadline
        with pytest.raises(expected):
            transmitter.ping()

    @pytest.mark.parametrize(
        ["host", "count", "expected"],
        [
            ["localhost", 0, ValueError],
            ["localhost", -1, ValueError],
            ["localhost", "a", ValueError],
        ],
    )
    def test_except_count(self, transmitter, host, count, expected):
        transmitter.destination_host = host
        transmitter.count = count
        with pytest.raises(expected):
            transmitter.ping()



# encoding: utf-8

"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

from __future__ import absolute_import, division

import abc

import six


@six.add_metaclass(abc.ABCMeta)
class PingParserInterface(object):
    @abc.abstractmethod
    def parse(self, ping_message):  # pragma: no cover
        pass



# encoding: utf-8

"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

from __future__ import absolute_import

from .__version__ import __author__, __copyright__, __email__, __license__, __version__
from ._logger import set_log_level, set_logger
from ._pingparsing import PingParsing
from ._pingtransmitter import PingResult, PingTransmitter
from ._stats import PingStats
from .error import ParseError



# encoding: utf-8

"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

from __future__ import absolute_import

import ipaddress
import platform
import warnings
from collections import namedtuple

import six
import subprocrunner
import typepy
from typepy import Integer

from ._logger import logger


DEFAULT_DEADLINE = 3


class PingResult(namedtuple("PingResult", "stdout stderr returncode")):
    """
    Data class to store ``ping`` command execution result.

    .. py:attribute:: stdout

        Standard output of ``ping`` command execution result.

    .. py:attribute:: stderr

        Standard error of ``ping`` command execution result.

    .. py:attribute:: returncode

        Return code of ``ping`` command execution result.
    """


class PingTransmitter(object):
    """
    Transmitter class to send ICMP packets by using the OS built-in ``ping``
    command.

    .. py:attribute:: destination_host

        Hostname or IP-address (IPv4/IPf6) to sending ICMP packets.

    .. py:attribute:: deadline

        Time ``[sec]`` of sending ICMP packets. The attribute ignored if
        the value is ``None``. If both :py:attr:`~.deadline` and
        :py:attr:`~.count` are ``None``, :py:attr:`~.deadline` set to ``1``.
        Defaults to ``None``.

    .. py:attribute:: count

        Number of sending ICMP packets. This attribute ignored if the value is
        ``None``. Defaults to ``None``.

    .. py:attribute:: ping_option

        Additional ``ping`` command option.

    .. py:attribute:: interface

        Interface name or zone-id. The attribute required when
        :py:attr:`~.destination_host` is IPv6 link-local scope address.
        Defaults to ``None``.

    .. py:attribute:: auto_codepage

        [Only for Windows environment] Automatically change code page if
        ``True``. Defaults to ``True``.
    """

    @property
    def waittime(self):
        warnings.warn(
            "waittime will be deleted in the future, use deadline instead.", DeprecationWarning
        )

        return self.deadline

    @waittime.setter
    def waittime(self, value):
        warnings.warn(
            "waittime will be deleted in the future, use deadline instead.", DeprecationWarning
        )

        self.deadline = value

    def __init__(self):
        self.destination_host = ""
        self.deadline = None
        self.count = None
        self.ping_option = ""
        self.is_quiet = False
        self.interface = None
        self.auto_codepage = True

    def ping(self):
        """
        Sending ICMP packets.

        :return: ``ping`` command execution result.
        :rtype: :py:class:`.PingResult`
        :raises ValueError: If parameters not valid.
        """

        self.__validate_ping_param()

        ping_proc = subprocrunner.SubprocessRunner(self.__get_ping_command())
        ping_proc.run()

        return PingResult(ping_proc.stdout, ping_proc.stderr, ping_proc.returncode)

    @staticmethod
    def __is_linux():
        return platform.system() == "Linux"

    @staticmethod
    def __is_macos():
        return platform.system() == "Darwin"

    @staticmethod
    def __is_windows():
        return platform.system() == "Windows"

    def __is_ipv6(self):
        try:
            network = ipaddress.ip_address(six.text_type(self.destination_host))
        except ValueError as e:
            logger.debug(e)
            return False

        logger.debug(
            "IP address: version={}, address={}".format(network.version, self.destination_host)
        )

        return network.version == 6

    def __validate_ping_param(self):
        if typepy.is_null_string(self.destination_host):
            raise ValueError("required destination_host")

        self.__validate_deadline()
        self.__validate_count()
        self.__validate_interface()

    def __validate_deadline(self):
        if self.deadline is None:
            return

        try:
            deadline = Integer(self.deadline).convert()
        except typepy.TypeConversionError:
            raise ValueError("deadline must be an integer: actual={}".format(self.deadline))

        if deadline <= 0:
            raise ValueError("deadline must be greater than zero: actual={}".format(self.deadline))

    def __validate_count(self):
        if self.count is None:
            return

        try:
            count = Integer(self.count).convert()
        except typepy.TypeConversionError:
            raise ValueError("count must be an integer: actual={}".format(self.count))

        if count <= 0:
            raise ValueError("count must be greater than zero")

    def __validate_interface(self):
        if not self.__is_ipv6():
            return

        if not ipaddress.ip_network(six.text_type(self.destination_host)).is_link_local:
            return

        if typepy.is_null_string(self.interface):
            raise ValueError("interface required to ping to IPv6 link local address")

    def __get_ping_command(self):
        command_list = []

        if self.__is_windows() and self.auto_codepage:
            command_list.append("chcp 437 &")

        command_list.extend(
            [
                self.__get_builtin_ping_command(),
                self.__get_deadline_option(),
                self.__get_count_option(),
                self.__get_quiet_option(),
            ]
        )

        if self.__is_linux() and typepy.is_not_null_string(self.interface):
            command_list.append("-I {}".format(self.interface))

        if typepy.is_not_null_string(self.ping_option):
            command_list.append(self.ping_option)

        command_list.append(self.__get_destination_host())

        return " ".join(command_list)

    def __get_destination_host(self):
        if self.__is_windows() and self.__is_ipv6():
            return "{:s}%{}".format(self.destination_host, self.interface)

        return self.destination_host

    def __get_builtin_ping_command(self):
        if self.__is_windows():
            return "ping"

        if self.__is_ipv6():
            return "ping6"

        return "ping"

    def __get_quiet_option(self):
        if not self.is_quiet or self.__is_windows():
            return ""

        return "-q"

    def __get_deadline_option(self):
        try:
            deadline = Integer(self.deadline).convert()
        except typepy.TypeConversionError:
            if self.count:
                return ""

            deadline = DEFAULT_DEADLINE

        if self.__is_windows():
            # ping for Windows not have the option with equals to the deadline
            # option.
            return "-n {:d}".format(deadline)
        elif self.__is_macos():
            if self.__is_ipv6():
                # there is no timeout option for macOS ping6.
                # so, using -i and -c option to simulate timeout.
                return "-i 1 -c {:d}".format(deadline)

            return "-t {:d}".format(deadline)

        return "-w {:d}".format(deadline)

    def __get_count_option(self):
        try:
            count = Integer(self.count).convert()
        except typepy.TypeConversionError:
            return ""

        if self.__is_windows():
            return "-n {:d}".format(count)

        return "-c {:d}".format(count)



# encoding: utf-8

"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

from __future__ import absolute_import

import pytest
from pingparsing import PingResult

from .common import ping_parser  # noqa: W0611
from .data import DEBIAN_SUCCESS_0


class Test_PingResult(object):
    @pytest.mark.parametrize(
        ["pingresult", "expected"],
        [[PingResult(DEBIAN_SUCCESS_0.value, "", 0), DEBIAN_SUCCESS_0.expected]],
    )
    def test_normal_pingresult(self, ping_parser, pingresult, expected):
        ping_parser.parse(pingresult.stdout)
        assert ping_parser.as_dict() == expected

        ping_parser.parse(pingresult)
        assert ping_parser.as_dict() == expected



# encoding: utf-8

"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

from __future__ import absolute_import

from collections import namedtuple

import pytest
from pingparsing import PingParsing


@pytest.fixture
def ping_parser():
    return PingParsing()


PingTestData = namedtuple("PingTestData", "value expected reply")



# encoding: utf-8

"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

from __future__ import absolute_import, division

import abc
import re
from datetime import datetime

import pyparsing as pp
import typepy
from typepy import Integer

from ._common import _to_unicode
from ._interface import PingParserInterface
from ._logger import logger
from ._stats import PingStats
from .error import ParseError, ParseErrorReason


class PingParser(PingParserInterface):

    _IPADDR_PATTERN = r"(\d{1,3}\.){3}\d{1,3}"
    _ICMP_SEQ_PATTERN = r"icmp_seq=(?P<icmp_seq>\d+)"
    _TTL_PATTERN = r"ttl=(?P<ttl>\d+)"
    _TIME_PATTERN = r"time=(?P<time>[0-9\.]+)"

    @abc.abstractproperty
    def _parser_name(self):  # pragma: no cover
        pass

    @abc.abstractproperty
    def _icmp_reply_pattern(self):  # pragma: no cover
        pass

    @property
    def _duplicate_packet_pattern(self):
        return r".+ \(DUP!\)$"

    @abc.abstractproperty
    def _stats_headline_pattern(self):  # pragma: no cover
        pass

    @abc.abstractproperty
    def _is_support_packet_duplicate(self):  # pragma: no cover
        pass

    def _parse_icmp_reply(self, ping_line_list):
        icmp_reply_regexp = re.compile(self._icmp_reply_pattern, re.IGNORECASE)
        duplicate_packet_regexp = re.compile(self._duplicate_packet_pattern)
        icmp_reply_list = []

        for line in ping_line_list:
            match = icmp_reply_regexp.search(line)
            if not match:
                continue

            reply = match.groupdict()

            if reply.get("timestamp"):
                reply["timestamp"] = datetime.fromtimestamp(
                    Integer(reply["timestamp"].lstrip("[").rstrip("]")).force_convert()
                )

            if reply.get("icmp_seq"):
                reply["icmp_seq"] = int(reply["icmp_seq"])

            if reply.get("ttl"):
                reply["ttl"] = int(reply["ttl"])

            if reply.get("time"):
                reply["time"] = float(reply["time"])

            if duplicate_packet_regexp.search(line):
                reply["duplicate"] = True
            else:
                reply["duplicate"] = False

            icmp_reply_list.append(reply)

        return icmp_reply_list

    def _preprocess_parse_stats(self, line_list):
        logger.debug("parsing as {:s} ping result format".format(self._parser_name))

        stats_headline_idx = self.__find_stats_headline_idx(
            line_list, re.compile(self._stats_headline_pattern)
        )
        body_line_list = line_list[stats_headline_idx + 1 :]
        self.__validate_stats_body(body_line_list)

        packet_info_line = body_line_list[0]

        return (line_list[stats_headline_idx], packet_info_line, body_line_list)

    def _parse_destination(self, stats_headline):
        return stats_headline.lstrip("--- ").rstrip(" ping statistics ---")

    def __find_stats_headline_idx(self, line_list, re_stats_header):
        for i, line in enumerate(line_list):
            if re_stats_header.search(line):
                break
        else:
            raise ParseError(reason=ParseErrorReason.HEADER_NOT_FOUND)

        return i

    def __validate_stats_body(self, body_line_list):
        if typepy.is_empty_sequence(body_line_list):
            raise ParseError(reason=ParseErrorReason.EMPTY_STATISTICS)

    def _parse_duplicate(self, line):
        if not self._is_support_packet_duplicate:
            return None

        packet_pattern = (
            pp.SkipTo(pp.Word("+" + pp.nums) + pp.Literal("duplicates,"))
            + pp.Word("+" + pp.nums)
            + pp.Literal("duplicates,")
        )

        try:
            duplicate_parse_list = packet_pattern.parseString(_to_unicode(line))
        except pp.ParseException:
            return 0

        return int(duplicate_parse_list[-2].strip("+"))


class NullPingParser(PingParser):
    @property
    def _parser_name(self):
        return "null"

    @property
    def _icmp_reply_pattern(self):
        return ""

    @property
    def _stats_headline_pattern(self):
        return ""

    @property
    def _is_support_packet_duplicate(self):  # pragma: no cover
        return False

    def parse(self, ping_message):  # pragma: no cover
        pass

    def _preprocess_parse_stats(self, line_list):  # pragma: no cover
        pass


class LinuxPingParser(PingParser):
    @property
    def _parser_name(self):
        return "Linux"

    @property
    def _icmp_reply_pattern(self):
        return (
            r"(?P<timestamp>\[[0-9\.]+\])?\s?.+ from "
            + self._IPADDR_PATTERN
            + ".*"
            + self._ICMP_SEQ_PATTERN
            + " "
            + self._TTL_PATTERN
            + " "
            + self._TIME_PATTERN
        )

    @property
    def _stats_headline_pattern(self):
        return "--- .* ping statistics ---"

    @property
    def _is_support_packet_duplicate(self):
        return True

    def parse(self, ping_message):
        icmp_reply_list = self._parse_icmp_reply(ping_message)
        stats_headline, packet_info_line, body_line_list = self._preprocess_parse_stats(
            line_list=ping_message
        )
        packet_pattern = (
            pp.Word(pp.nums)
            + pp.Literal("packets transmitted,")
            + pp.Word(pp.nums)
            + pp.Literal("received,")
        )

        destination = self._parse_destination(stats_headline)
        duplicates = self._parse_duplicate(packet_info_line)

        parse_list = packet_pattern.parseString(_to_unicode(packet_info_line))
        packet_transmit = int(parse_list[0])
        packet_receive = int(parse_list[2])

        is_valid_data = True
        try:
            rtt_line = body_line_list[1]
        except IndexError:
            is_valid_data = False

        if not is_valid_data or typepy.is_null_string(rtt_line):
            return PingStats(
                destination=destination,
                packet_transmit=packet_transmit,
                packet_receive=packet_receive,
                duplicates=duplicates,
                icmp_reply_list=icmp_reply_list,
            )

        rtt_pattern = (
            pp.Literal("rtt min/avg/max/mdev =")
            + pp.Word(pp.nums + ".")
            + "/"
            + pp.Word(pp.nums + ".")
            + "/"
            + pp.Word(pp.nums + ".")
            + "/"
            + pp.Word(pp.nums + ".")
            + pp.Word(pp.nums + "ms")
        )
        parse_list = rtt_pattern.parseString(_to_unicode(rtt_line))

        return PingStats(
            destination=destination,
            packet_transmit=packet_transmit,
            packet_receive=packet_receive,
            duplicates=duplicates,
            rtt_min=float(parse_list[1]),
            rtt_avg=float(parse_list[3]),
            rtt_max=float(parse_list[5]),
            rtt_mdev=float(parse_list[7]),
            icmp_reply_list=icmp_reply_list,
        )


class WindowsPingParser(PingParser):
    @property
    def _parser_name(self):
        return "Windows"

    @property
    def _icmp_reply_pattern(self):
        return " from " + self._IPADDR_PATTERN + ".*" + self._TTL_PATTERN + " " + self._TIME_PATTERN

    @property
    def _stats_headline_pattern(self):
        return "^Ping statistics for "

    @property
    def _is_support_packet_duplicate(self):
        return False

    def parse(self, ping_message):
        icmp_reply_list = self._parse_icmp_reply(ping_message)
        stats_headline, packet_info_line, body_line_list = self._preprocess_parse_stats(
            line_list=ping_message
        )
        packet_pattern = (
            pp.Literal("Packets: Sent = ")
            + pp.Word(pp.nums)
            + pp.Literal(", Received = ")
            + pp.Word(pp.nums)
        )

        destination = self._parse_destination(stats_headline)
        duplicates = self._parse_duplicate(packet_info_line)

        parse_list = packet_pattern.parseString(_to_unicode(packet_info_line))
        packet_transmit = int(parse_list[1])
        packet_receive = int(parse_list[3])

        is_valid_data = True
        try:
            rtt_line = body_line_list[2].strip()
        except IndexError:
            is_valid_data = False

        if not is_valid_data or typepy.is_null_string(rtt_line):
            return PingStats(
                destination=destination,
                packet_transmit=packet_transmit,
                packet_receive=packet_receive,
                duplicates=duplicates,
                icmp_reply_list=icmp_reply_list,
            )

        rtt_pattern = (
            pp.Literal("Minimum = ")
            + pp.Word(pp.nums)
            + pp.Literal("ms, Maximum = ")
            + pp.Word(pp.nums)
            + pp.Literal("ms, Average = ")
            + pp.Word(pp.nums)
        )
        parse_list = rtt_pattern.parseString(_to_unicode(rtt_line))

        return PingStats(
            destination=destination,
            packet_transmit=packet_transmit,
            packet_receive=packet_receive,
            duplicates=duplicates,
            rtt_min=float(parse_list[1]),
            rtt_avg=float(parse_list[5]),
            rtt_max=float(parse_list[3]),
            icmp_reply_list=icmp_reply_list,
        )

    def _parse_destination(self, stats_headline):
        return stats_headline.lstrip("Ping statistics for ").rstrip(":")


class MacOsPingParser(PingParser):
    @property
    def _parser_name(self):
        return "macOS"

    @property
    def _icmp_reply_pattern(self):
        return (
            " from "
            + self._IPADDR_PATTERN
            + ".*"
            + self._ICMP_SEQ_PATTERN
            + " "
            + self._TTL_PATTERN
            + " "
            + self._TIME_PATTERN
        )

    @property
    def _stats_headline_pattern(self):
        return "--- .* ping statistics ---"

    @property
    def _is_support_packet_duplicate(self):
        return True

    def parse(self, ping_message):
        icmp_reply_list = self._parse_icmp_reply(ping_message)
        stats_headline, packet_info_line, body_line_list = self._preprocess_parse_stats(
            line_list=ping_message
        )
        packet_pattern = (
            pp.Word(pp.nums)
            + pp.Literal("packets transmitted,")
            + pp.Word(pp.nums)
            + pp.Literal("packets received,")
        )

        destination = self._parse_destination(stats_headline)
        duplicates = self._parse_duplicate(packet_info_line)

        parse_list = packet_pattern.parseString(_to_unicode(packet_info_line))
        packet_transmit = int(parse_list[0])
        packet_receive = int(parse_list[2])

        is_valid_data = True
        try:
            rtt_line = body_line_list[1]
        except IndexError:
            is_valid_data = False

        if not is_valid_data or typepy.is_null_string(rtt_line):
            return PingStats(
                destination=destination,
                packet_transmit=packet_transmit,
                packet_receive=packet_receive,
                duplicates=duplicates,
                icmp_reply_list=icmp_reply_list,
            )

        rtt_pattern = (
            pp.Literal("round-trip min/avg/max/stddev =")
            + pp.Word(pp.nums + ".")
            + "/"
            + pp.Word(pp.nums + ".")
            + "/"
            + pp.Word(pp.nums + ".")
            + "/"
            + pp.Word(pp.nums + ".")
            + pp.Word(pp.nums + "ms")
        )
        parse_list = rtt_pattern.parseString(_to_unicode(rtt_line))

        return PingStats(
            destination=destination,
            packet_transmit=packet_transmit,
            packet_receive=packet_receive,
            duplicates=duplicates,
            rtt_min=float(parse_list[1]),
            rtt_avg=float(parse_list[3]),
            rtt_max=float(parse_list[5]),
            rtt_mdev=float(parse_list[7]),
            icmp_reply_list=icmp_reply_list,
        )


class AlpineLinuxPingParser(LinuxPingParser):
    @property
    def _parser_name(self):
        return "AlpineLinux"

    @property
    def _icmp_reply_pattern(self):
        return (
            " from "
            + self._IPADDR_PATTERN
            + ".*"
            + r"seq=(?P<icmp_seq>\d+) "
            + self._TTL_PATTERN
            + " "
            + self._TIME_PATTERN
        )

    @property
    def _is_support_packet_duplicate(self):
        return True

    def parse(self, ping_message):
        icmp_reply_list = self._parse_icmp_reply(ping_message)
        stats_headline, packet_info_line, body_line_list = self._preprocess_parse_stats(
            line_list=ping_message
        )
        packet_pattern = (
            pp.Word(pp.nums)
            + pp.Literal("packets transmitted,")
            + pp.Word(pp.nums)
            + pp.Literal("packets received,")
        )

        destination = self._parse_destination(stats_headline)
        duplicates = self._parse_duplicate(packet_info_line)

        parse_list = packet_pattern.parseString(_to_unicode(packet_info_line))
        packet_transmit = int(parse_list[0])
        packet_receive = int(parse_list[2])

        is_valid_data = True
        try:
            rtt_line = body_line_list[1]
        except IndexError:
            is_valid_data = False

        if not is_valid_data or typepy.is_null_string(rtt_line):
            return PingStats(
                destination=destination,
                packet_transmit=packet_transmit,
                packet_receive=packet_receive,
                duplicates=duplicates,
                icmp_reply_list=icmp_reply_list,
            )

        rtt_pattern = (
            pp.Literal("round-trip min/avg/max =")
            + pp.Word(pp.nums + ".")
            + "/"
            + pp.Word(pp.nums + ".")
            + "/"
            + pp.Word(pp.nums + ".")
            + pp.Word(pp.nums + "ms")
        )
        parse_list = rtt_pattern.parseString(_to_unicode(rtt_line))

        return PingStats(
            destination=destination,
            packet_transmit=packet_transmit,
            packet_receive=packet_receive,
            duplicates=duplicates,
            rtt_min=float(parse_list[1]),
            rtt_avg=float(parse_list[3]),
            rtt_max=float(parse_list[5]),
            icmp_reply_list=icmp_reply_list,
        )

    def _parse_duplicate(self, line):
        packet_pattern = (
            pp.SkipTo(pp.Word(pp.nums) + pp.Literal("duplicates,"))
            + pp.Word(pp.nums)
            + pp.Literal("duplicates,")
        )
        try:
            duplicate_parse_list = packet_pattern.parseString(_to_unicode(line))
        except pp.ParseException:
            return 0

        return int(duplicate_parse_list[-2])



#!/usr/bin/env python
# encoding: utf-8

"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

from __future__ import print_function

import argparse
import multiprocessing
import os
import sys
from textwrap import dedent

import logbook
import pingparsing
from subprocrunner import CommandError

from .__version__ import __version__


try:
    import simplejson as json
except ImportError:
    import json


DEFAULT_COUNT = 10
QUIET_LOG_LEVEL = logbook.NOTSET


def parse_option():
    parser = argparse.ArgumentParser(
        formatter_class=argparse.RawDescriptionHelpFormatter,
        epilog=dedent(
            """\
            Documentation: https://pingparsing.rtfd.io/
            Issue tracker: https://github.com/thombashi/pingparsing/issues
            """
        ),
    )
    parser.add_argument(
        "-V", "--version", action="version", version="%(prog)s {}".format(__version__)
    )

    if is_use_stdin():
        parser.add_argument("destination_or_file", nargs="+", help="")

    parser.add_argument(
        "--max-workers",
        type=int,
        help="""a number of threads for when multiple destination/file
        specified. defaults to equals to two times number of cores.
        """,
    )
    parser.add_argument(
        "--indent",
        type=int,
        default=4,
        help="""JSON output will be pretty-printed with the indent level.
        (default= %(default)s)
        """,
    )
    parser.add_argument(
        "--icmp-reply",
        action="store_true",
        default=False,
        help="print results for each ICMP packet reply.",
    )

    loglevel_dest = "log_level"
    group = parser.add_mutually_exclusive_group()
    group.add_argument(
        "--debug",
        dest=loglevel_dest,
        action="store_const",
        const=logbook.DEBUG,
        default=logbook.INFO,
        help="for debug print.",
    )
    group.add_argument(
        "--quiet",
        dest=loglevel_dest,
        action="store_const",
        const=QUIET_LOG_LEVEL,
        default=logbook.INFO,
        help="suppress execution log messages.",
    )

    group = parser.add_argument_group("Ping Options")
    group.add_argument(
        "-c",
        "--count",
        type=int,
        help="""stop after sending the count.
        see also ping(8) [-c count] option description.
        """,
    )
    group.add_argument(
        "-w",
        "--deadline",
        type=float,
        help="""timeout in seconds.
        see also ping(8) [-w deadline] option description.
        """,
    )
    group.add_argument("-I", "--interface", dest="interface", help="network interface")

    return parser.parse_args()


def initialize_log_handler(log_level):
    debug_level_format_str = (
        "[{record.level_name}] {record.channel} {record.func_name} "
        "({record.lineno}): {record.message}"
    )
    if log_level == logbook.DEBUG:
        info_level_format_str = debug_level_format_str
    else:
        info_level_format_str = "[{record.level_name}] {record.channel}: {record.message}"

    logbook.StderrHandler(
        level=logbook.DEBUG, format_string=debug_level_format_str
    ).push_application()
    logbook.StderrHandler(
        level=logbook.INFO, format_string=info_level_format_str
    ).push_application()


def is_use_stdin():
    return sys.stdin.isatty() or len(sys.argv) > 1


def parse_ping(logger, dest_or_file, interface, count, deadline, is_parse_icmp_reply):
    if os.path.isfile(dest_or_file):
        with open(dest_or_file) as f:
            ping_result_text = f.read()
    else:
        transmitter = pingparsing.PingTransmitter()
        transmitter.destination_host = dest_or_file
        transmitter.interface = interface
        transmitter.count = count
        transmitter.deadline = deadline
        transmitter.is_quiet = not is_parse_icmp_reply

        try:
            result = transmitter.ping()
        except CommandError as e:
            logger.error(e)
            sys.exit(e.errno)

        ping_result_text = result.stdout
        if result.returncode != 0:
            logger.error(result.stderr)

    ping_parser = pingparsing.PingParsing()
    stats = ping_parser.parse(ping_result_text)
    output = stats.as_dict()
    if is_parse_icmp_reply:
        output["icmp_reply"] = stats.icmp_reply_list

    return (dest_or_file, output)


def get_ping_param(options):
    count = options.count
    deadline = options.deadline

    if not options.count and not options.deadline:
        count = DEFAULT_COUNT

    return (count, deadline)


def print_result(text):
    if not sys.stdout.isatty():
        # avoid to colorized when piped or redirected
        print(text)
        return

    try:
        from pygments import highlight
        from pygments.lexers import JsonLexer
        from pygments.formatters import TerminalTrueColorFormatter

        print(
            highlight(
                code=text, lexer=JsonLexer(), formatter=TerminalTrueColorFormatter(style="monokai")
            ).strip()
        )
    except ImportError:
        print(text)


def main():
    options = parse_option()

    initialize_log_handler(options.log_level)

    logger = logbook.Logger("pingparsing cli")
    logger.level = options.log_level
    pingparsing.set_log_level(options.log_level)

    output = {}
    if is_use_stdin():
        from concurrent import futures

        pingparsing.set_log_level(options.log_level)

        max_workers = (
            multiprocessing.cpu_count() * 2 if options.max_workers is None else options.max_workers
        )
        count, deadline = get_ping_param(options)
        logger.debug("max-workers={}, count={}, deadline={}".format(max_workers, count, deadline))

        try:
            with futures.ProcessPoolExecutor(max_workers) as executor:
                future_list = []
                for dest_or_file in options.destination_or_file:
                    logger.debug("start {}".format(dest_or_file))
                    future_list.append(
                        executor.submit(
                            parse_ping,
                            logger,
                            dest_or_file,
                            options.interface,
                            count,
                            deadline,
                            options.icmp_reply,
                        )
                    )

                for future in futures.as_completed(future_list):
                    key, ping_data = future.result()
                    output[key] = ping_data
        finally:
            logger.debug("shutdown ProcessPoolExecutor")
            executor.shutdown()
    else:
        ping_result_text = sys.stdin.read()
        ping_parser = pingparsing.PingParsing()
        stats = ping_parser.parse(ping_result_text)
        output = stats.as_dict()
        if options.icmp_reply:
            output["icmp_reply"] = stats.icmp_reply_list

    if options.indent <= 0:
        result = json.dumps(output)
    else:
        result = json.dumps(output, indent=options.indent)

    print_result(result)

    return 0


if __name__ == "__main__":
    sys.exit(main())



# encoding: utf-8

"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

from __future__ import absolute_import, division, unicode_literals


class PingStats(object):
    def __init__(self, *args, **kwargs):
        self.__destination = kwargs.pop("destination", None)
        self.__packet_transmit = kwargs.pop("packet_transmit", None)
        self.__packet_receive = kwargs.pop("packet_receive", None)
        self.__rtt_min = kwargs.pop("rtt_min", None)
        self.__rtt_avg = kwargs.pop("rtt_avg", None)
        self.__rtt_max = kwargs.pop("rtt_max", None)
        self.__rtt_mdev = kwargs.pop("rtt_mdev", None)
        self.__duplicates = kwargs.pop("duplicates", None)

        self.__icmp_reply_list = kwargs.pop("icmp_reply_list", [])

    @property
    def destination(self):
        """
        :return: The ping destination.
        :rtype: str
        """

        return self.__destination

    @property
    def packet_transmit(self):
        """
        :return: Number of packets transmitted.
        :rtype: int
        """

        return self.__packet_transmit

    @property
    def packet_receive(self):
        """
        :return: Number of packets received.
        :rtype: int
        """

        return self.__packet_receive

    @property
    def packet_loss_count(self):
        """
        :return: Packet loss count. ``None`` if the value is not a number.
        :rtype: int
        """

        try:
            return self.packet_transmit - self.packet_receive
        except TypeError:
            return None

    @property
    def packet_loss_rate(self):
        """
        :return:
            Percentage of packet loss ``[%]``.
            ``None`` if the value is not a number.
        :rtype: float
        """

        try:
            return (self.packet_loss_count / self.packet_transmit) * 100
        except (TypeError, ZeroDivisionError, OverflowError):
            return None

    @property
    def rtt_min(self):
        """
        :return:
            Minimum round trip time of transmitted ICMP packets ``[msec]``.
        :rtype: float
        """

        return self.__rtt_min

    @property
    def rtt_avg(self):
        """
        :return:
            Average round trip time of transmitted ICMP packets ``[msec]``.
        :rtype: float
        """

        return self.__rtt_avg

    @property
    def rtt_max(self):
        """
        :return:
            Maximum round trip time of transmitted ICMP packets ``[msec]``.
        :rtype: float
        """

        return self.__rtt_max

    @property
    def rtt_mdev(self):
        """
        :return:
            Standard deviation of transmitted ICMP packets. The attribute returns
            always ``None`` when parsing Windows ping result.
        :rtype: float
        """

        return self.__rtt_mdev

    @property
    def packet_duplicate_count(self):
        """
        :return:
            Number of duplicated packet. The attribute returns always ``None`` when parsing
            Windows ping result.
        :rtype: int
        """

        return self.__duplicates

    @property
    def packet_duplicate_rate(self):
        """
        :return:
            Percentage of duplicated packets ``[%]``. ``None`` if the value is not a number.
        :rtype: float
        """

        try:
            return (self.packet_duplicate_count / self.packet_receive) * 100
        except (TypeError, ZeroDivisionError, OverflowError):
            return None

    @property
    def icmp_reply_list(self):
        """
        :return: List of ICMP packet reply information.
        :rtype: list of dict
        """

        return self.__icmp_reply_list

    def as_dict(self):
        """
        :return: ping statistics.
        :rtype: dict
        """

        return {
            "destination": self.destination,
            "packet_transmit": self.packet_transmit,
            "packet_receive": self.packet_receive,
            "packet_loss_count": self.packet_loss_count,
            "packet_loss_rate": self.packet_loss_rate,
            "rtt_min": self.rtt_min,
            "rtt_avg": self.rtt_avg,
            "rtt_max": self.rtt_max,
            "rtt_mdev": self.rtt_mdev,
            "packet_duplicate_count": self.packet_duplicate_count,
            "packet_duplicate_rate": self.packet_duplicate_rate,
        }

    def as_tuple(self):
        """
        :return: ping statistics.
        :rtype: collections.namedtuple
        """

        from collections import namedtuple

        ping_result = self.as_dict()

        return namedtuple("PingStatsTuple", ping_result.keys())(**ping_result)



# encoding: utf-8

"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

from __future__ import absolute_import, unicode_literals

import logbook
import subprocrunner


logger = logbook.Logger("pingparsing lib")
logger.disable()


def set_logger(is_enable):
    if is_enable != logger.disabled:
        # logger setting have not changed
        return

    if is_enable:
        logger.enable()
    else:
        logger.disable()

    subprocrunner.set_logger(is_enable)


def set_log_level(log_level):
    """
    Set logging level of this module. The module using
    `logbook <https://logbook.readthedocs.io/en/stable/>`__ module for logging.

    :param int log_level:
        One of the log level of the
        `logbook <https://logbook.readthedocs.io/en/stable/api/base.html>`__.
        Disabled logging if the ``log_level`` is ``logbook.NOTSET``.
    :raises LookupError: If ``log_level`` is an invalid value.
    """

    # validate log level
    logbook.get_level_name(log_level)

    if log_level == logger.level:
        return

    if log_level == logbook.NOTSET:
        set_logger(is_enable=False)
    else:
        set_logger(is_enable=True)

    logger.level = log_level
    subprocrunner.set_log_level(log_level)



# encoding: utf-8

"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

from __future__ import print_function, unicode_literals

import pytest
import simplejson as json
from subprocrunner import SubprocessRunner

from .data import DEBIAN_SUCCESS_0, WINDOWS7SP1_SUCCESS


def print_result(stdout, stderr):
    print("[stdout]\n{}".format(stdout))
    print("[stderr]\n{}".format(stderr))


@pytest.mark.xfail(run=False)
class Test_cli_file(object):
    def test_normal_single(self, tmpdir):
        tmp_ping_file = tmpdir.join("ping_deb.txt")
        tmp_ping_file.write(DEBIAN_SUCCESS_0.value)
        tmp_ping_path = str(tmp_ping_file)

        runner = SubprocessRunner(["pingparsing", tmp_ping_path])
        runner.run()

        print_result(stdout=runner.stdout, stderr=runner.stderr)

        assert runner.returncode == 0
        assert json.loads(runner.stdout)[tmp_ping_path] == DEBIAN_SUCCESS_0.expected

    def test_normal_multi(self, tmpdir):
        tmp_ping_file_deb = tmpdir.join("ping_deb.txt")
        tmp_ping_file_deb.write(DEBIAN_SUCCESS_0.value)
        tmp_ping_path_deb = str(tmp_ping_file_deb)

        tmp_ping_file_win = tmpdir.join("ping_win.txt")
        tmp_ping_file_win.write(WINDOWS7SP1_SUCCESS.value)
        tmp_ping_path_win = str(tmp_ping_file_win)

        runner = SubprocessRunner(["pingparsing", tmp_ping_path_deb, tmp_ping_path_win])
        runner.run()

        print_result(stdout=runner.stdout, stderr=runner.stderr)

        assert runner.returncode == 0

        parsed_result = json.loads(runner.stdout)
        assert parsed_result[tmp_ping_path_deb] == DEBIAN_SUCCESS_0.expected
        assert parsed_result[tmp_ping_path_win] == WINDOWS7SP1_SUCCESS.expected


@pytest.mark.xfail(run=False)
class Test_cli_pipe(object):
    def test_normal_single(self, tmpdir):
        runner = SubprocessRunner(["pingparsing"])
        runner.run(input=DEBIAN_SUCCESS_0.value)

        print_result(stdout=runner.stdout, stderr=runner.stderr)

        assert runner.returncode == 0
        assert json.loads(runner.stdout) == DEBIAN_SUCCESS_0.expected


@pytest.mark.xfail(run=False)
class Test_PingParsing_ping(object):
    def test_normal_single(self):
        count = 1
        dest = "localhost"
        runner = SubprocessRunner(["pingparsing", dest, "-c", count])
        runner.run()

        print_result(stdout=runner.stdout, stderr=runner.stderr)

        assert runner.returncode == 0

        parsed_result = json.loads(runner.stdout)

        assert parsed_result[dest]["packet_transmit"] == count
        assert parsed_result[dest]["rtt_max"] > 0

    def test_normal_multi(self):
        count = 1
        dest_list = ["google.com", "twitter.com"]
        runner = SubprocessRunner(["pingparsing"] + dest_list + ["-c", count])
        runner.run()

        print_result(stdout=runner.stdout, stderr=runner.stderr)

        assert runner.returncode == 0

        parsed_result = json.loads(runner.stdout)
        for dest in dest_list:
            assert parsed_result[dest]["packet_transmit"] == count
            assert parsed_result[dest]["rtt_max"] > 0



# encoding: utf-8

"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

from __future__ import absolute_import, division

import pyparsing as pp
import typepy

from ._common import _to_unicode
from ._interface import PingParserInterface
from ._logger import logger
from ._parser import (
    AlpineLinuxPingParser,
    LinuxPingParser,
    MacOsPingParser,
    NullPingParser,
    WindowsPingParser,
)
from ._stats import PingStats
from .error import ParseError, ParseErrorReason


class PingParsing(PingParserInterface):
    """
    Parser class to parsing ping command output.
    """

    def __init__(self):
        self.__parser = NullPingParser()
        self.__stats = None

    @property
    def parser_name(self):
        return self.__parser._parser_name

    @property
    def destination(self):
        """
        (Duprecated)

        :return: The ping destination.
        :rtype: str
        """

        return self.__stats.destination

    @property
    def packet_transmit(self):
        """
        (Duprecated)

        :return: Number of packets transmitted.
        :rtype: int
        """

        return self.__stats.packet_transmit

    @property
    def packet_receive(self):
        """
        (Duprecated)

        :return: Number of packets received.
        :rtype: int
        """

        return self.__stats.packet_receive

    @property
    def packet_loss_rate(self):
        """
        (Duprecated)

        :return:
            Percentage of packet loss ``[%]``.
            ``None`` if the value is not a number.
        :rtype: float
        """

        return self.__stats.packet_loss_rate

    @property
    def packet_loss(self):
        # mark as delete
        return self.packet_loss_rate

    @property
    def packet_loss_count(self):
        """
        (Duprecated)

        :return: Packet loss count. ``None`` if the value is not a number.
        :rtype: int
        """

        return self.__stats.packet_loss_count

    @property
    def rtt_min(self):
        """
        (Duprecated)

        :return:
            Minimum round trip time of transmitted ICMP packets ``[msec]``.
        :rtype: float
        """

        return self.__stats.rtt_min

    @property
    def rtt_avg(self):
        """
        (Duprecated)

        :return:
            Average round trip time of transmitted ICMP packets ``[msec]``.
        :rtype: float
        """

        return self.__stats.rtt_avg

    @property
    def rtt_max(self):
        """
        (Duprecated)

        :return:
            Maximum round trip time of transmitted ICMP packets ``[msec]``.
        :rtype: float
        """

        return self.__stats.rtt_max

    @property
    def rtt_mdev(self):
        """
        (Duprecated)

        :return:
            Standard deviation of transmitted ICMP packets. The attribute
            returns always ``None`` when parsing Windows ping result.
        :rtype: float
        """

        return self.__stats.rtt_mdev

    @property
    def packet_duplicate_rate(self):
        """
        (Duprecated)

        :return:
            Percentage of duplicated packets ``[%]``.
            ``None`` if the value is not a number.
        :rtype: float
        """

        return self.__stats.packet_duplicate_rate

    @property
    def packet_duplicate_count(self):
        """
        (Duprecated)

        :return:
            Number of duplicated packet. The attribute
            returns always ``None`` when parsing Windows ping result.
        :rtype: int
        """

        return self.__stats.packet_duplicate_count

    @property
    def duplicates(self):
        # mark as delete
        return self.packet_duplicate_count

    def as_dict(self):
        """
        (Duprecated)

        :return: Parsed result as a dictionary.
        :rtype: dict

        :Examples:
            >>> import pingparsing
            >>> parser = pingparsing.PingParsing()
            >>> parser.parse(ping_result)
            >>> parser.as_dict()
            {
                "destination": "google.com",
                "packet_transmit": 60,
                "packet_receive": 60,
                "packet_loss_rate": 0.0,
                "packet_loss_count": 0,
                "rtt_min": 61.425,
                "rtt_avg": 99.731,
                "rtt_max": 212.597,
                "rtt_mdev": 27.566,
                "packet_duplicate_rate": 0.0,
                "packet_duplicate_count": 0
            }
        """

        return self.__stats.as_dict()

    def as_tuple(self):
        """
        (Duprecated)

        :return: Parsed result as a tuple.
        :rtype: collections.namedtuple

        :Examples:
            >>> import pingparsing
            >>> parser = pingparsing.PingParsing()
            >>> parser.parse(ping_result)
            >>> parser.as_tuple()
            PingResult(destination='google.com', packet_transmit=60, packet_receive=60, packet_loss_rate=0.0, packet_loss_count=0, rtt_min=61.425, rtt_avg=99.731, rtt_max=212.597, rtt_mdev=27.566, packet_duplicate_rate=0.0, packet_duplicate_count=0)
        """

        return self.__stats.as_tuple()

    def parse(self, ping_message):
        """
        Parse ping command output.

        :param ping_message: ping command output.
        :type ping_message: str or :py:class:`~pingparsing.PingResult`
        :return: Parsed result.

            .. seealso:: :py:meth:`.as_tuple`
        :rtype: :py:class:`~pingparsing.PingStats`
        """

        try:
            # accept PingResult instance as an input
            if typepy.is_not_null_string(ping_message.stdout):
                ping_message = ping_message.stdout
        except AttributeError:
            pass

        logger.debug("parsing ping result: {}".format(ping_message))

        self.__parser = NullPingParser()

        if typepy.is_null_string(ping_message):
            logger.debug("ping_message is empty")
            self.__stats = PingStats()

            return self.__stats

        line_list = _to_unicode(ping_message).splitlines()
        parser_class_list = (
            LinuxPingParser,
            WindowsPingParser,
            MacOsPingParser,
            AlpineLinuxPingParser,
        )

        for parser_class in parser_class_list:
            self.__parser = parser_class()
            try:
                self.__stats = self.__parser.parse(line_list)
                return self.__stats
            except ParseError as e:
                if e.reason != ParseErrorReason.HEADER_NOT_FOUND:
                    raise e
            except pp.ParseException:
                pass

        self.__parser = NullPingParser()

        return self.__stats



# encoding: utf-8

"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

from __future__ import absolute_import

from textwrap import dedent

import pytest
import six
from pingparsing import ParseError

from .common import PingTestData, ping_parser  # noqa: W0611
from .data import DEBIAN_SUCCESS_0, UBUNTU_SUCCESS_0, UBUNTU_SUCCESS_1, WINDOWS7SP1_SUCCESS


PING_FEDORA_EMPTY_BODY = six.b(
    dedent(
        """\
    PING 192.168.0.1 (192.168.0.1) 56(84) bytes of data.

    --- 192.168.0.1 ping statistics ---
    """
    )
)
PING_WINDOWS_INVALID = dedent(
    """\
    Pinging 192.168.207.100 with 32 bytes of data:
    Request timed out.
    Request timed out.
    Request timed out.
    Request timed out.

    Ping statistics for 192.168.207.100:
    """
)

DEBIAN_UNREACHABLE_0 = PingTestData(
    dedent(
        """\
        PING 192.168.207.100 (192.168.207.100) 56(84) bytes of data.

        --- 192.168.207.100 ping statistics ---
        5 packets transmitted, 0 received, 100% packet loss, time 4009ms
        """
    ),
    {
        "destination": "192.168.207.100",
        "packet_transmit": 5,
        "packet_receive": 0,
        "packet_duplicate_count": 0,
        "packet_duplicate_rate": None,
        "packet_loss_count": 5,
        "packet_loss_rate": 100.0,
        "rtt_min": None,
        "rtt_avg": None,
        "rtt_max": None,
        "rtt_mdev": None,
    },
    [],
)
DEBIAN_UNREACHABLE_1 = PingTestData(
    DEBIAN_UNREACHABLE_0.value + "\n", DEBIAN_UNREACHABLE_0.expected, []
)
DEBIAN_UNREACHABLE_2 = PingTestData(
    DEBIAN_UNREACHABLE_1.value + "\n", DEBIAN_UNREACHABLE_0.expected, []
)

FEDORA_DUP_LOSS = PingTestData(
    dedent(
        """\
        PING 192.168.0.1 (192.168.0.1) 56(84) bytes of data.

        --- 192.168.0.1 ping statistics ---
        1688 packets transmitted, 1553 received, +1 duplicates, 7% packet loss, time 2987ms
        rtt min/avg/max/mdev = 0.282/0.642/11.699/0.699 ms, pipe 2, ipg/ewma 1.770/0.782 ms
        """
    ),
    {
        "destination": "192.168.0.1",
        "packet_transmit": 1688,
        "packet_receive": 1553,
        "packet_duplicate_count": 1,
        "packet_duplicate_rate": 0.0643915003219575,
        "packet_loss_count": 135,
        "packet_loss_rate": 7.9976303317535535,
        "rtt_min": 0.282,
        "rtt_max": 11.699,
        "rtt_mdev": 0.699,
        "rtt_avg": 0.642,
    },
    [],
)
FEDORA_UNREACHABLE = PingTestData(
    dedent(
        """\
        PING 192.168.207.100 (192.168.207.100) 56(84) bytes of data.
        From 192.168.207.128 icmp_seq=1 Destination Host Unreachable
        From 192.168.207.128 icmp_seq=2 Destination Host Unreachable
        From 192.168.207.128 icmp_seq=3 Destination Host Unreachable
        From 192.168.207.128 icmp_seq=4 Destination Host Unreachable
        From 192.168.207.128 icmp_seq=5 Destination Host Unreachable

        --- 192.168.207.100 ping statistics ---
        5 packets transmitted, 0 received, +5 errors, 100% packet loss, time 4003ms
        """
    ),
    {
        "destination": "192.168.207.100",
        "packet_transmit": 5,
        "packet_receive": 0,
        "packet_loss_count": 5,
        "packet_loss_rate": 100.0,
        "packet_duplicate_count": 0,
        "packet_duplicate_rate": None,
        "rtt_min": None,
        "rtt_avg": None,
        "rtt_max": None,
        "rtt_mdev": None,
    },
    [],
)

MACOS_SUCCESS_0 = PingTestData(
    dedent(
        """\
        PING google.com (172.217.6.238): 56 data bytes
        64 bytes from 172.217.6.238: icmp_seq=0 ttl=53 time=20.482 ms
        64 bytes from 172.217.6.238: icmp_seq=1 ttl=53 time=32.550 ms
        64 bytes from 172.217.6.238: icmp_seq=2 ttl=53 time=32.013 ms
        64 bytes from 172.217.6.238: icmp_seq=3 ttl=53 time=28.498 ms
        64 bytes from 172.217.6.238: icmp_seq=4 ttl=53 time=46.093 ms

        --- google.com ping statistics ---
        5 packets transmitted, 5 packets received, 0.0% packet loss
        round-trip min/avg/max/stddev = 20.482/31.927/46.093/8.292 ms
        """
    ),
    {
        "destination": "google.com",
        "packet_transmit": 5,
        "packet_receive": 5,
        "packet_loss_count": 0,
        "packet_loss_rate": 0.0,
        "packet_duplicate_count": 0,
        "packet_duplicate_rate": 0,
        "rtt_min": 20.482,
        "rtt_avg": 31.927,
        "rtt_max": 46.093,
        "rtt_mdev": 8.292,
    },
    [
        {"icmp_seq": 0, "ttl": 53, "time": 20.482, "duplicate": False},
        {"icmp_seq": 1, "ttl": 53, "time": 32.55, "duplicate": False},
        {"icmp_seq": 2, "ttl": 53, "time": 32.013, "duplicate": False},
        {"icmp_seq": 3, "ttl": 53, "time": 28.498, "duplicate": False},
        {"icmp_seq": 4, "ttl": 53, "time": 46.093, "duplicate": False},
    ],
)
MACOS_SUCCESS_1 = PingTestData(
    dedent(
        """\
        PING github.com (192.30.255.113): 56 data bytes

        --- github.com ping statistics ---
        10 packets transmitted, 10 packets received, 0.0% packet loss
        round-trip min/avg/max/stddev = 218.391/283.477/405.879/70.170 ms
        """
    ),
    {
        "destination": "github.com",
        "packet_transmit": 10,
        "packet_receive": 10,
        "packet_loss_count": 0,
        "packet_loss_rate": 0.0,
        "packet_duplicate_count": 0,
        "packet_duplicate_rate": 0,
        "rtt_min": 218.391,
        "rtt_avg": 283.477,
        "rtt_max": 405.879,
        "rtt_mdev": 70.170,
    },
    [],
)
MACOS_UNREACHABLE_0 = PingTestData(
    dedent(
        """\
        PING twitter.com (59.24.3.173): 56 data bytes
        ^C
        --- twitter.com ping statistics ---
        59 packets transmitted, 0 packets received, 100.0% packet loss
        """
    ),
    {
        "destination": "twitter.com",
        "packet_transmit": 59,
        "packet_receive": 0,
        "packet_loss_rate": 100.0,
        "packet_loss_count": 59,
        "rtt_min": None,
        "rtt_avg": None,
        "rtt_max": None,
        "rtt_mdev": None,
        "packet_duplicate_rate": None,
        "packet_duplicate_count": 0,
    },
    [],
)
MACOS_UNREACHABLE_1 = PingTestData(
    dedent(
        """\
        PING twitter.com (31.13.78.66): 56 data bytes

        --- twitter.com ping statistics ---
        10 packets transmitted, 0 packets received, 100.0% packet loss
        """
    ),
    {
        "destination": "twitter.com",
        "packet_transmit": 10,
        "packet_receive": 0,
        "packet_loss_count": 10,
        "packet_loss_rate": 100.0,
        "packet_duplicate_count": 0,
        "packet_duplicate_rate": None,
        "rtt_min": None,
        "rtt_avg": None,
        "rtt_max": None,
        "rtt_mdev": None,
    },
    [],
)
MACOS_DUPLICATE_0 = PingTestData(
    dedent(
        """\
        PING duplicate.com (31.13.78.66): 56 data bytes

        --- duplicate.com ping statistics ---
        3 packets transmitted, 3 packets received, +3 duplicates, 0% packet loss

        round-trip min/avg/max/stddev = 0.311/1.091/2.186/0.662 ms
        """
    ),
    {
        "destination": "duplicate.com",
        "packet_transmit": 3,
        "packet_receive": 3,
        "packet_loss_count": 0,
        "packet_loss_rate": 0.0,
        "packet_duplicate_count": 3,
        "packet_duplicate_rate": 100,
        "rtt_min": None,
        "rtt_avg": None,
        "rtt_max": None,
        "rtt_mdev": None,
    },
    [],
)

ALPINE_LINUX_SUCCESS = PingTestData(
    dedent(
        """\
        PING heise.de (193.99.144.80): 56 data bytes

        --- heise.de ping statistics ---
        5 packets transmitted, 5 packets received, 0% packet loss
        round-trip min/avg/max = 0.638/0.683/0.746 ms
        """
    ),
    {
        "destination": "heise.de",
        "packet_transmit": 5,
        "packet_receive": 5,
        "packet_loss_count": 0,
        "packet_loss_rate": 0.0,
        "packet_duplicate_count": 0,
        "packet_duplicate_rate": 0,
        "rtt_min": 0.638,
        "rtt_avg": 0.683,
        "rtt_max": 0.746,
        "rtt_mdev": None,
    },
    [],
)
ALPINE_LINUX_DUP_LOSS = PingTestData(
    dedent(
        """\
        PING 192.168.2.106 (192.168.2.106): 56 data bytes
        64 bytes from 192.168.2.106: seq=0 ttl=64 time=0.936 ms
        64 bytes from 192.168.2.106: seq=0 ttl=64 time=1.003 ms (DUP!)
        64 bytes from 192.168.2.106: seq=1 ttl=64 time=0.802 ms
        64 bytes from 192.168.2.106: seq=2 ttl=64 time=0.696 ms
        64 bytes from 192.168.2.106: seq=3 ttl=64 time=0.664 ms
        64 bytes from 192.168.2.106: seq=4 ttl=64 time=1.194 ms
        64 bytes from 192.168.2.106: seq=5 ttl=64 time=0.613 ms
        64 bytes from 192.168.2.106: seq=6 ttl=64 time=0.898 ms
        64 bytes from 192.168.2.106: seq=8 ttl=64 time=1.066 ms
        64 bytes from 192.168.2.106: seq=9 ttl=64 time=1.144 ms
        64 bytes from 192.168.2.106: seq=9 ttl=64 time=1.219 ms (DUP!)

        --- 192.168.2.106 ping statistics ---
        10 packets transmitted, 9 packets received, 2 duplicates, 10% packet loss
        round-trip min/avg/max = 0.613/0.930/1.219 ms
        """
    ),
    {
        "destination": "192.168.2.106",
        "packet_transmit": 10,
        "packet_receive": 9,
        "packet_duplicate_count": 2,
        "packet_duplicate_rate": 22.22222222222222,
        "packet_loss_count": 1,
        "packet_loss_rate": 10.0,
        "rtt_min": 0.613,
        "rtt_avg": 0.93,
        "rtt_max": 1.219,
        "rtt_mdev": None,
    },
    [
        {"icmp_seq": 0, "ttl": 64, "time": 0.936, "duplicate": False},
        {"icmp_seq": 0, "ttl": 64, "time": 1.003, "duplicate": True},
        {"icmp_seq": 1, "ttl": 64, "time": 0.802, "duplicate": False},
        {"icmp_seq": 2, "ttl": 64, "time": 0.696, "duplicate": False},
        {"icmp_seq": 3, "ttl": 64, "time": 0.664, "duplicate": False},
        {"icmp_seq": 4, "ttl": 64, "time": 1.194, "duplicate": False},
        {"icmp_seq": 5, "ttl": 64, "time": 0.613, "duplicate": False},
        {"icmp_seq": 6, "ttl": 64, "time": 0.898, "duplicate": False},
        {"icmp_seq": 8, "ttl": 64, "time": 1.066, "duplicate": False},
        {"icmp_seq": 9, "ttl": 64, "time": 1.144, "duplicate": False},
        {"icmp_seq": 9, "ttl": 64, "time": 1.219, "duplicate": True},
    ],
)

WINDOWS10_LOSS = PingTestData(
    dedent(
        """\
        Pinging 192.168.2.106 with 32 bytes of data:
        Reply from 192.168.2.106: bytes=32 time=16ms TTL=64
        Reply from 192.168.2.106: bytes=32 time=6ms TTL=64
        Reply from 192.168.2.106: bytes=32 time=12ms TTL=64
        Reply from 192.168.2.106: bytes=32 time=16ms TTL=64
        Request timed out.
        Reply from 192.168.2.106: bytes=32 time=8ms TTL=64
        Reply from 192.168.2.106: bytes=32 time=33ms TTL=64
        Reply from 192.168.2.106: bytes=32 time=13ms TTL=64
        Reply from 192.168.2.106: bytes=32 time=23ms TTL=64
        Reply from 192.168.2.106: bytes=32 time<1ms TTL=64

        Ping statistics for 192.168.2.106:
            Packets: Sent = 10, Received = 9, Lost = 1 (10% los
        Approximate round trip times in milli-seconds:
            Minimum = 0ms, Maximum = 33ms, Average = 14ms
        """
    ),
    {
        "destination": "192.168.2.106",
        "packet_transmit": 10,
        "packet_receive": 9,
        "packet_loss_count": 1,
        "packet_loss_rate": 10.0,
        "packet_duplicate_count": None,
        "packet_duplicate_rate": None,
        "rtt_min": 0.0,
        "rtt_avg": 14.0,
        "rtt_max": 33.0,
        "rtt_mdev": None,
    },
    [],
)
WINDOWS_UNREACHABLE_0 = PingTestData(
    dedent(
        """\
        Pinging 192.168.207.100 with 32 bytes of data:
        Request timed out.
        Request timed out.
        Request timed out.
        Request timed out.

        Ping statistics for 192.168.207.100:
            Packets: Sent = 4, Received = 0, Lost = 4 (100% loss),
        """
    ),
    {
        "destination": "192.168.207.100",
        "packet_transmit": 4,
        "packet_receive": 0,
        "packet_loss_count": 4,
        "packet_loss_rate": 100.0,
        "packet_duplicate_count": None,
        "packet_duplicate_rate": None,
        "rtt_min": None,
        "rtt_avg": None,
        "rtt_max": None,
        "rtt_mdev": None,
    },
    [],
)
WINDOWS_UNREACHABLE_1 = PingTestData(
    WINDOWS_UNREACHABLE_0.value + "\n", WINDOWS_UNREACHABLE_0.expected, []
)
WINDOWS_UNREACHABLE_2 = PingTestData(
    WINDOWS_UNREACHABLE_1.value + "\n", WINDOWS_UNREACHABLE_0.expected, []
)


class Test_PingParsing_parse(object):
    @pytest.mark.parametrize(
        ["test_data", "parser_name"],
        [
            [DEBIAN_SUCCESS_0, "Linux"],
            [DEBIAN_UNREACHABLE_0, "Linux"],
            [DEBIAN_UNREACHABLE_1, "Linux"],
            [DEBIAN_UNREACHABLE_2, "Linux"],
            [UBUNTU_SUCCESS_0, "Linux"],
            [FEDORA_DUP_LOSS, "Linux"],
            [FEDORA_UNREACHABLE, "Linux"],
            [MACOS_SUCCESS_0, "macOS"],
            [MACOS_SUCCESS_1, "macOS"],
            [MACOS_UNREACHABLE_0, "macOS"],
            [MACOS_UNREACHABLE_1, "macOS"],
            [MACOS_DUPLICATE_0, "macOS"],
            [ALPINE_LINUX_SUCCESS, "AlpineLinux"],
            [ALPINE_LINUX_DUP_LOSS, "AlpineLinux"],
            [WINDOWS7SP1_SUCCESS, "Windows"],
            [WINDOWS10_LOSS, "Windows"],
            [WINDOWS_UNREACHABLE_0, "Windows"],
            [WINDOWS_UNREACHABLE_1, "Windows"],
            [WINDOWS_UNREACHABLE_2, "Windows"],
        ],
    )
    def test_normal_text(self, ping_parser, test_data, parser_name):
        stats = ping_parser.parse(test_data.value)

        print("[input text]\n{}\n".format(test_data.value))
        print("[expected]\n{}\n".format(test_data.expected))
        print("[actual]\n{}\n".format(stats.as_dict()))
        for icmp_reply in stats.icmp_reply_list:
            print(icmp_reply)

        assert ping_parser.parser_name == parser_name
        assert stats.as_dict() == test_data.expected
        assert stats.icmp_reply_list == test_data.reply

    @pytest.mark.xfail(run=False)
    @pytest.mark.parametrize(["test_data", "parser_name"], [[UBUNTU_SUCCESS_1, "Linux"]])
    def test_normal_timestamp(self, ping_parser, test_data, parser_name):
        stats = ping_parser.parse(test_data.value)

        print("[input text]\n{}\n".format(test_data.value))
        print("[expected]\n{}\n".format(test_data.expected))
        print("[actual]\n{}\n".format(stats.as_dict()))
        for icmp_reply in stats.icmp_reply_list:
            print(icmp_reply)

        assert ping_parser.parser_name == parser_name
        assert stats.as_dict() == test_data.expected
        assert stats.icmp_reply_list == test_data.reply

    def test_empty(self, ping_parser):
        ping_parser.parse(
            dedent(
                """\
            PING google.com (216.58.196.238) 56(84) bytes of data.

            --- google.com ping statistics ---
            60 packets transmitted, 60 received, 0% packet loss, time 59153ms
            rtt min/avg/max/mdev = 61.425/99.731/212.597/27.566 ms
            """
            )
        )
        stats = ping_parser.parse("")

        assert stats.destination is None
        assert stats.packet_transmit is None
        assert stats.packet_receive is None
        assert stats.packet_loss_count is None
        assert stats.packet_loss_rate is None
        assert stats.packet_duplicate_count is None
        assert stats.packet_duplicate_rate is None
        assert stats.rtt_min is None
        assert stats.rtt_avg is None
        assert stats.rtt_max is None
        assert stats.rtt_mdev is None
        assert stats.icmp_reply_list == []

    @pytest.mark.parametrize(
        ["value", "expected"],
        [[PING_FEDORA_EMPTY_BODY, ParseError], [PING_WINDOWS_INVALID, ParseError]],
    )
    def test_exception(self, ping_parser, value, expected):
        with pytest.raises(expected):
            ping_parser.parse(value)


class Test_PingParsing_as_tuple(object):
    def test_normal(self, ping_parser):
        stats = ping_parser.parse(DEBIAN_SUCCESS_0.value)
        result = stats.as_tuple()

        assert result.destination == "google.com"
        assert result.packet_transmit == 60
        assert result.packet_receive == 60
        assert result.packet_loss_count == 0
        assert result.packet_loss_rate == 0.0
        assert result.packet_duplicate_rate == 0
        assert result.packet_duplicate_count == 0
        assert result.rtt_min == 61.425
        assert result.rtt_avg == 99.731
        assert result.rtt_max == 212.597
        assert result.rtt_mdev == 27.566
        assert stats.icmp_reply_list == []



#!/usr/bin/env python
# encoding: utf-8

"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

from __future__ import print_function

import json


def print_ping_parser(ping_parser):
    print("# properties ---")
    print("packet_transmit: {:d} packets".format(ping_parser.packet_transmit))
    print("packet_receive: {:d} packets".format(ping_parser.packet_receive))
    print("packet_loss_rate: {:.1f} %".format(ping_parser.packet_loss_rate))
    print("packet_loss_count: {:d} packets".format(ping_parser.packet_loss_count))

    if ping_parser.packet_duplicate_rate:
        packet_duplicate_rate = "{:.1f} %".format(ping_parser.packet_duplicate_rate)
    else:
        packet_duplicate_rate = "NaN"
    print("packet_duplicate_rate: {:s}".format(packet_duplicate_rate))

    if ping_parser.packet_duplicate_count:
        packet_duplicate_count = "{:d} packets".format(ping_parser.packet_duplicate_count)
    else:
        packet_duplicate_count = "NaN"
    print("packet_duplicate_count: {:s}".format(packet_duplicate_count))

    print("rtt_min:", ping_parser.rtt_min)
    print("rtt_avg:", ping_parser.rtt_avg)
    print("rtt_max:", ping_parser.rtt_max)
    print("rtt_mdev:", ping_parser.rtt_mdev)
    print()
    print("# as_dict ---")
    print(json.dumps(ping_parser.as_dict(), indent=4))



# encoding: utf-8

"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

from __future__ import absolute_import

import enum


@enum.unique
class ParseErrorReason(enum.Enum):
    HEADER_NOT_FOUND = "ping statistics not found"
    EMPTY_STATISTICS = "ping statistics is empty"


class ParseError(Exception):
    """
    Exception raised when failed to parse ping results.
    """

    @property
    def reason(self):
        return self.__reason

    def __init__(self, *args, **kwargs):
        self.__reason = kwargs.pop("reason", None)

        super(ParseError, self).__init__(*args, **kwargs)



# encoding: utf-8

"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

from __future__ import absolute_import


def _to_unicode(text):
    try:
        return text.decode("ascii")
    except AttributeError:
        return text
